#include "kernel/types.h"
#include "user/user.h"
#include "kernel/fcntl.h"
#include "kernel/fs.h"
#include "kernel/stat.h"

#define MAX_PATH 128

// 查找文件的函数
void find(char *path, char *name) {
    char buf[MAX_PATH];
    char fullpath[MAX_PATH];
    char originpath[MAX_PATH];
    int fd;
    struct dirent de;
    struct stat st;

    strncpy(fullpath, path, MAX_PATH - 1);
    
    fullpath[MAX_PATH - 1] = '\0'; // 确保字符串以空字符终止
    int len = strlen(fullpath);
    if (len + 1 < MAX_PATH) { // 确保有足够的空间添加 "/"
        fullpath[len] = '/';
        fullpath[len + 1] = '\0';
    }

    //保存原来的路径，递归的时候使用这个路径
    strncpy(originpath, path, MAX_PATH - 1);
    originpath[MAX_PATH - 1] = '\0'; // 确保字符串以空字符终止
    int len1 = strlen(originpath);
    if (len1 + 1 < MAX_PATH) { // 确保有足够的空间添加 "/"
        originpath[len1] = '/';
        originpath[len1 + 1] = '\0';
    }

    // 打开目录
    fd = open(fullpath, O_RDONLY);
    if (fd < 0) {
        fprintf(2, "find: cannot open %s\n", path);
        return;
    }

    // 读取目录内容
    while (read(fd, &de, sizeof(de)) > 0) {

        if (de.inum == 0)
            continue;

        // 构建文件或目录的完整路径
        memset(buf, 0, MAX_PATH);
        strncpy(buf, originpath, MAX_PATH - 1);
        buf[MAX_PATH - 1] = '\0'; // 确保字符串以空字符终止
        int buf_len = strlen(buf);

        
        if (buf_len + 1 + strlen(de.name) < MAX_PATH) { // 确保有足够的空间添加名称
        // 如果不是根目录才增加一个斜杠
            if (strcmp(path, ".") != 0){
                buf[buf_len] = '/';
            }
            strncpy(buf + buf_len + 0, de.name, MAX_PATH - buf_len - 1);//拼接字符串
            buf[MAX_PATH - 1] = '\0'; // 确保字符串以空字符终止
        }


        // 获取文件状态
        if (stat(buf, &st) < 0) {
            fprintf(2, "find: cannot stat %s\n", buf);
            continue;
        }

        // 检查名称是否匹配
        if (strcmp(de.name, name) == 0) {
            printf("%s\n", buf);
        }

        // 如果是目录，则递归查找
        if (st.type == T_DIR && strcmp(de.name, ".") != 0 && strcmp(de.name, "..") != 0) {
            strncpy(fullpath, buf, MAX_PATH - 1);
            fullpath[MAX_PATH - 1] = '\0'; // 确保字符串以空字符终止
            find(fullpath, name);
        }
    }
    close(fd);
}

int main(int argc, char *argv[]) {
    if (argc < 3) {
        fprintf(2, "Usage: find <path> <name>\n");
        exit(1);
    }

    find(argv[1], argv[2]);
    exit(0);
}