#include "kernel/types.h"
#include "user/user.h"
#include "kernel/fcntl.h"
#include "kernel/stat.h"

#define PIPE_READ 0
#define PIPE_WRITE 1

int main(void) {
    int c2f[2], f2c[2];
    int pid;
    char buf[512];
    int n;

    // 创建两个管道
    if (pipe(c2f) < 0 || pipe(f2c) < 0) {
        fprintf(2, "pipe() failed\n");
        exit(1);
    }

    pid = fork();
    if (pid < 0) {
        fprintf(2, "fork() failed\n");
        exit(1);
    } else if (pid == 0) { // 子进程
        // 关闭子进程不需要的管道端
        close(c2f[PIPE_READ]);
        close(f2c[PIPE_WRITE]);

        // 从父进程读取数据
        n = read(f2c[PIPE_READ], buf, sizeof(buf));
        if (n > 0) {
            // 子进程通过管道读取父进程的 PID
            int parent_pid = *(int*)buf;
            fprintf(1, "%d: received ping from pid %d\n", getpid(), parent_pid);
            // 向父进程发送数据
            n = write(c2f[PIPE_WRITE], "pong", 5);
            if (n < 0) {
                fprintf(2, "child write to pipe failed\n");
            }
        }
        exit(0);
    } else { // 父进程
        // 关闭父进程不需要的管道端
        close(c2f[PIPE_WRITE]);
        close(f2c[PIPE_READ]);

        // 将父进程的 PID 写入管道
        int parent_pid = getpid();
        if (write(f2c[PIPE_WRITE], &parent_pid, sizeof(parent_pid)) != sizeof(parent_pid)) {
            fprintf(2, "parent write to pipe failed\n");
        } else {
            // 向子进程发送数据
            n = write(f2c[PIPE_WRITE], "ping", 4);
            if (n < 0) {
                fprintf(2, "parent write to pipe failed\n");
            } else {
                // 从子进程读取数据
                n = read(c2f[PIPE_READ], buf, sizeof(buf));
                if (n > 0) {
                    fprintf(1, "%d: received pong from pid %d\n", getpid(), pid);
                }
            }
        }
        wait(0); // 等待子进程退出
    }
    exit(0);
}
